/*
    Valtra Tractors
    Copyright 2004 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Martin Storsj
    martin@martin.st
*/

#include <SDL.h>
#include <SDL_image.h>
#include "wrapper.h"
#include "effects.h"
#include "fsys.h"


#define DIALOGFADE 100

Game *constructWrapper(SDL_Surface *screen, Uint32 time) {
	return new Wrapper(screen,time);
}

Wrapper::Wrapper(SDL_Surface *screen, Uint32 time) : GameWrapper(screen,time) {
	boardSurface = NULL;
	boardGame = NULL;
	front = IMG_Load("front.png");
	SDL_SetAlpha(front,SDL_RLEACCEL,0);
	startTime = time;
	lastTime = time;

	pause = false;
	pointer[0] = IMG_Load("pointer1.png");
	pointer[1] = IMG_Load("pointer2.png");
	pointer[2] = IMG_Load("pointer3.png");
	setPointer(0);
	SDL_GetMouseState(&curMousePos.x,&curMousePos.y);
	dialog = NULL;
	screenSize.x = screen->w;
	screenSize.y = screen->h;
	quit = false;
	fade = lastFade = 0;
}

Wrapper::~Wrapper() {
	delete boardGame;
	SDL_FreeSurface(boardSurface);
	SDL_FreeSurface(front);
	for (int i = 0; i < 3; i++)
		SDL_FreeSurface(pointer[i]);
	if (dialog) delete dialog;
}

void Wrapper::handleEvent(SDL_Event &event) {
	if (event.type==SDL_KEYDOWN) {
		if (boardGame==NULL && dialog==NULL) {
			showDialog(PLAYERS);
			return;
		}
		if (dialog && fade!=0) {
			if (fade==1) fade = 0;
			else startTime = lastTime-DIALOGFADE;
			return;
		}
		if (event.key.keysym.sym==SDLK_p) {
			if (boardGame) {
				pause = !pause;
				boardGame->pause(pause);
			}
			return;
		} else if (event.key.keysym.sym==SDLK_q) {
			quit = true;
			return;
		}
	} else if (event.type==SDL_MOUSEMOTION) {
		curMousePos.x = event.motion.x;
		curMousePos.y = event.motion.y;
	} else if (event.type==SDL_MOUSEBUTTONDOWN) {
		if (boardGame==NULL && dialog==NULL) {
			showDialog(PLAYERS);
			return;
		}
		if (dialog && fade!=0) {
			if (fade==1) fade = 0;
			else startTime = lastTime-DIALOGFADE;
			return;
		}
	}

	if (dialog) {
		if (event.type==SDL_MOUSEMOTION) {
			event.motion.x -= dialogPos.x;
			event.motion.y -= dialogPos.y;
		} else if (event.type==SDL_MOUSEBUTTONDOWN || event.type==SDL_MOUSEBUTTONUP) {
			event.button.x -= dialogPos.x;
			event.button.y -= dialogPos.y;
		}
		if (fade==0) dialog->handleEvent(event);
	} else if (boardGame) boardGame->handleEvent(event);
}

void Wrapper::showDialog(DialogType type, int data) {
	if (dialog!=NULL) delete dialog;
	dialog = new Dialog(type,data);
	dType = type;
	dialog->setWrapper(this);
	SDL_Surface *surf = dialog->getSurface();
	dialogPos.x = (screenSize.x-surf->w)/2;
	dialogPos.y = (screenSize.y-surf->h)/2;
	startTime = lastTime;
	fade = 1;
}

void Wrapper::initGame(SDL_Surface *screen, int time, int players) {
	if (boardGame==NULL) {
		boardSurface = SDL_CreateRGBSurface(SDL_HWSURFACE,screen->w,screen->h,32,screen->format->Rmask,screen->format->Gmask,screen->format->Bmask,0);
		boardGame = constructBoardGame(boardSurface,time);
		boardGame->setWrapper(this);
		SDL_FreeSurface(front);
		front = NULL;
	}
	boardGame->startGame(players);
	setPointer(0);
}

void Wrapper::update(Uint32 time, SDL_Surface *screen) {
	lastTime = time;
	if (boardGame) {
		if (boardGame->finished()) {
			if (!dialog) {
//				showDialog(NEWGAME);
				showDialog(RESULT,boardGame->getResult());
			}
		}
	} else {
		if (dialog==NULL && time-startTime>5000) {
			showDialog(PLAYERS);
		}
	}

	if (dialog && dialog->finished()) {
		if (fade==0) {
			setPointer(0);
			fade = 2; 
			startTime = time;
		}
	}
	if (dialog && fade==2 && time-startTime>=DIALOGFADE) {
		int result = dialog->getResult();
		delete dialog;
		dialog = NULL;
		if (dType==QUIT) {
			if (result==0) {
//				quit = true;
//				showDialog(NEWGAME);
				showDialog(PLAYERS);
			} else boardGame->pause(false);
		} else if (dType==NEWGAME) {
			if (result==1) quit = true;
			else showDialog(PLAYERS);
		} else if (dType==PLAYERS) {
			if (result==0) quit = true;
			else {
				if (result==4) initGame(screen,time,4);
				else showDialog(CHOOSE,result);
			}
		} else if (dType==CHOOSE) {
			if (result==0) {
				showDialog(PLAYERS);
			} else {
				initGame(screen,time,result);
			}
		} else if (dType==RESULT) {
//			showDialog(NEWGAME);
			showDialog(PLAYERS);
		}
	}

	if (boardGame) boardGame->update(time,boardSurface);

	int pointerId = se.getNextId();

	vector<SDL_Rect> rects;
	if (boardGame) boardGame->getRects(rects);
	for (unsigned int i = 0; i < rects.size(); i++) {
		se.redrawRect(rects[i]);
	}

	int dialogId = se.getNextId();

	SDL_Surface *tempSurf = NULL;
	if (dialog) {
		dialog->update();
		dialog->getRects(rects);
		for (unsigned int i = 0; i < rects.size(); i++) {
			rects[i].x += dialogPos.x;
			rects[i].y += dialogPos.y;
			se.redrawRect(rects[i]);
		}
		SDL_Rect rect;
		SDL_Surface *surf = dialog->getSurface();
		rect.x = dialogPos.x;
		rect.y = dialogPos.y;
		rect.w = surf->w;
		rect.h = surf->h;
		if (fade==1)
			if (time-startTime>=DIALOGFADE)
				fade = 0;
#if DIALOGFADE>0
		if (fade==1) {
			tempSurf = Effects::multAlpha(surf,255*(time-startTime)/DIALOGFADE);
			se.addSprite(dialogId,new Sprite(tempSurf,rect,true));
		} else if (fade==2) {
			tempSurf = Effects::multAlpha(surf,255*(DIALOGFADE-(time-startTime))/DIALOGFADE);
			se.addSprite(dialogId,new Sprite(tempSurf,rect,true));
		} else
#endif
			se.addSprite(dialogId,new Sprite(surf,rect,fade!=lastFade));
		lastFade = fade;
	}


	if (dialog!=NULL || boardGame!=NULL) {
		SDL_Rect rect;
		rect.x = curMousePos.x - curPointerOffset.x;
		rect.y = curMousePos.y - curPointerOffset.y;
		rect.w = curPointer->w;
		rect.h = curPointer->h;
		bool redraw = false;
		if (curPointer!=lastPointer) redraw = true;
		lastPointer = curPointer;
		se.addSprite(pointerId,new Sprite(curPointer,rect,redraw));
	}


	se.calcUpdates();
	if (boardGame) se.update(screen,boardSurface);
	else se.update(screen,front);
	se.reset();

	if (tempSurf) SDL_FreeSurface(tempSurf);
}

bool Wrapper::finished() {
	return quit;
}

void Wrapper::setUpdatedRect(void (*func)(const SDL_Rect &r)) {
	Game::setUpdatedRect(func);
//	boardGame->setUpdatedRect(func);
	se.setUpdatedRect(func);
}

void Wrapper::redraw() {
	if (boardGame) boardGame->redraw();
	se.redrawAll();
}

void Wrapper::startGame(int players) {
//	boardGame->startGame(players);
	if (boardGame) boardGame->startGame(0);
//	showDialog(PLAYERS);
}

void Wrapper::setPointer(int n) {
	curPointer = pointer[n];
	curPointerOffset = pointerOff[n];
}

void Wrapper::showQuitDialog() {
	if (dialog!=NULL) return;
	showDialog(QUIT);
	boardGame->pause(true);
}


